//Здесь приведен конструктор, в котором прежде всего создается объект сокета (указатель m_pTcpSocket). Затем вызывается метод
//connectToHost() этого объекта, устанавливающий связь с сервером. Первым параметром в этот метод передается имя компьютера, а вторым
//— номер порта. Коммуникация сокетов асинхронна. Сокет отправляет сигнал connected(), как только будет создано соединение, а так же сигнал
//readyRead() при готовности предоставить данные для чтения. Мы соединяем эти сигналы со слотами slotConnected() и slotReadyRead(). В
//случае возникновения ошибок сокет отправляет сигнал error(), который мы соединяем со слотом slotError(), предназначенным для отображения
//ошибок.
//Затем создается пользовательский интерфейс программы, состоящий из надписи, кнопки, однострочного и многострочного текстовых полей.
//Сигнал clicked() виджета кнопки нажатия соединяется со слотом slotSendToServer() класса MyClient, ответственным за отправку данных на
//сервер. Для того чтобы к аналогичному действию приводило и нажатие клавиши <Enter>, мы соединяем сигнал returnPressed() виджета текстового
//поля (m_ptxtInput) с тем же слотом slotSendToServer().

MyClient::MyClient(const QString& strHost,
                   int            nPort,
                   QWidget* pwgt /*=0*/
                  ) : QWidget(pwgt)
                    , m_nNextBlockSize(0)
{

    m_pTcpSocket = new QTcpSocket(this);

    m_pTcpSocket->connectToHost(strHost, nPort);
    connect(m_pTcpSocket, SIGNAL(connected()), SLOT(slotConnected()));
    connect(m_pTcpSocket, SIGNAL(readyRead()), SLOT(slotReadyRead()));
    connect(m_pTcpSocket, SIGNAL(error(QAbstractSocket::SocketError)),
            this,         SLOT(slotError(QAbstractSocket::SocketError))
    );

    m_ptxtInfo = new QTextEdit;
    m_ptxtInput = new QLineEdit;

    m_ptxtInfo->setReadOnly(true);

    QPushButton* pcmd = new QPushButton("&Send");
    connect(pcmd, SIGNAL(clicked()), SLOT(slotSendToServer()));
    connect(m_ptxtInput, SIGNAL(returnPressed()),
            this,        SLOT(slotSendToServer())
    );

    //Layout setup
    QVBoxLayout* pvbxLayout = new QVBoxLayout;
    pvbxLayout->addWidget(new QLabel("<H1>Client</H1>"));
    pvbxLayout->addWidget(m_ptxtInfo);
    pvbxLayout->addWidget(m_ptxtInput);
    pvbxLayout->addWidget(pcmd);
    setLayout(pvbxLayout);
}

//Здесь приведен слот slotReadyToRead(), который вызывается при поступлении данных от сервера. Цикл for нужен потому, что с сервера не все
//данные могут прийти одновременно. Поэтому клиент, как и сервер, должен быть в состоянии получать как весь блок целиком, так и только часть
//блока или даже все блоки сразу. Каждый принятый блок начинается полем размера блока.
//Когда мы будем уверены, что блок получен целиком, можно без опасения использовать оператор >> объекта потока QDataStream (переменная in).
//Чтение данных из сокета осуществляется при помощи объекта потока данных. Полученная информация добавляется в виджет многострочного
//текстового поля (указатель m_ptxtInfo) с помощью метода append().
//В завершение анализа блока данных атрибуту m_nNextBlockSize присваиваем значение 0, которое говорит о том, что размер очередного блока
//данных неизвестен.

void MyClient::slotReadyRead()
{
        QDataStream in(m_pTcpSocket);
        in.setVersion(QDataStream::Qt_4_5);
        for (;;) {
            if (!m_nNextBlockSize) {
                if (m_pTcpSocket->bytesAvailable() < sizeof(quint16)) {
                    break;
                }
                in >> m_nNextBlockSize;
            }

            if (m_pTcpSocket->bytesAvailable() < m_nNextBlockSize) {
                break;
            }
            QTime time;
            QString str;
            in >> time >> str;
            m_ptxtInfo->append(time.toString() + " " + str);
            m_nNextBlockSize = 0;
    }
}


//Слот slotError() вызывается при возникновении ошибок. В нем мы преобразуем код ошибки в текст, а затем отображаем его в виджете
//многострочного текстового поля.

void MyClient::slotError(QAbstractSocket::SocketError err)
{
    QString strError =
        "Error: " + (err == QAbstractSocket::HostNotFoundError ?
                    "The host was not found." :
                     err == QAbstractSocket::RemoteHostClosedError ?
                    "The remote host is closed." :
                    err == QAbstractSocket::ConnectionRefusedError ?
                    "The connection was refused." :
                    QString(m_pTcpSocket->errorString())
                    );
    m_ptxtInfo->append(strError);
}

//Mетод отсылки запроса серверу. В нем используется тот же трюк, что и в аналогичном методе сервера, который уже был рассмотрен ранее.
//Единственное отличие от метода MyServer::sendToClient() заключается в том, что после отсылки данных мы дополнительно стираем текст в поле
//ввода сообщения, вызывая метод setText() с пустой строкой в качестве параметра (можно было бы также использо-вать метод clear()).


void MyClient::slotSendToServer()
{
    QByteArray arrBlock;
    QDataStream out(&arrBlock, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_5);
    out << quint16(0) << QTime::currentTime() << m_ptxtInput->text();

    out.device()->seek(0);
    out << quint16(arrBlock.size() — sizeof(quint16));

    m_pTcpSocket->write(arrBlock);
    m_ptxtInput->setText("");
}

//Как только связь с сервером будет установлена, вызывается метод slotConnected(). Этот слот в виджет текстового поля добавляет строку
//сообщения.

void MyClient::slotConnected()
{
    m_ptxtInfo->append("Received the connected() signal");
}
