UdpServer::UdpServer(QWidget* pwgt /*=0*/) : QTextEdit(pwgt)
{
    setWindowTitle("UdpServer");

    m_pudp = new QUdpSocket(this);

    QTimer* ptimer = new QTimer(this);
    ptimer->setInterval(500);
    ptimer->start();
    connect(ptimer, SIGNAL(timeout()), SLOT(slotSendDatagram()));
}


//Для того чтобы мы могли отличить приложение сервера от клиента в конструкторе класса UdpServer, мы задаем заголовок окна методом
//setWindowTitle()и создаем объект сокета QUdpSocket. Пересылка дейтаграмм будет осуществляться через определенные интервалы времени при
//помощи таймера, поэтому создаем таймер, устанавливаем в нем интервал равный 0,5 сек, запускаем его вызовом метода start() и соединяем со
//сло-том пересылки дейтаграмм slotSenDatagram().

void UdpServer::slotSendDatagram()
{
    QByteArray baDatagram;
    QDataStream out(&baDatagram, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_7);
    QDateTime dt = QDateTime::currentDateTime();
    append("Sent:" + dt.toString());
    out << dt;
    m_pudp->writeDatagram(baDatagram, QHostAddress::LocalHost, 2424);
}

//Mетод slotSendDatagram(), который формирует и пересылает дейтаграммы. Наша дейтаграмма будет содержать только текущую дату и время.
//Воспользуемся классом потока QDataStream для помещения данных в объект baDatagram. Пересылка дейтаграмм осуществляется при помощи метода
//writeDatagram(), в который мы передаем первым параметром данные дейтаграммы baDatagram, вторым и третьим передаем IP-адрес и номер порта
//партнера. В нашем случае не указываем IP-адрес, а используем константу QHostAddress::LocalHost, которая имеет строковое значение
//локального хоста 127.0.0.1.

UdpClient::UdpClient(QWidget* pwgt /*=0*/) : QTextEdit(pwgt)
{
    setWindowTitle("UdpClient");
    m_pudp = new QUdpSocket(this);
    m_pudp->bind(2424);
    connect(m_pudp, SIGNAL(readyRead()), SLOT(slotProcessDatagrams()));
}

//Для того чтобы отличать окно клиента от окна сервера, в конструкторе для окна приложения клиента задаем надпись "UdpClient"
//Создаем объект сокета QUdpSocket и при помощи метода bind() привязываем его к порту 2424. Поскольку мы не указали IP-адрес хоста, то для
//получения данных по умолчанию сокет будет использовать локальный хост. Для извлечения и отображения полученных дейтаграмм соединяем
//сигнал сокета readyRead() со слотом slotProcessDatagrams().

void UdpClient::slotProcessDatagrams()
{
    QByteArray baDatagram;
    do {
        baDatagram.resize(m_pudp->pendingDatagramSize());
        m_pudp->readDatagram(baDatagram.data(), baDatagram.size());
    } while(m_pudp->hasPendingDatagrams());

    QDateTime dateTime;
    QDataStream in(&baDatagram, QIODevice::ReadOnly);
    in.setVersion(QDataStream::Qt_4_7);
    in >> dateTime;
    append("Received:" + dateTime.toString());
}


//Oбъект класса QUdpSocket при получении дейтаграмм ставит в очередь поступившие дейтаграммы и дает возможность последовательного доступа к
//ним в порядке очередности. Сама же очередь обычно состоит только из одной дейтаграммы, но никогда нельзя исключать возможность и того,
//что отправитель может последовательно передать сразу нескольких дейтаграмм. Поэтому и нужен цикл do...while(), т. к. в этом случае
//необходимо проигнорировать все дейтаграммы, кроме самой последней, поскольку именно она и содержит самые актуальные данные. В теле цикла
//при помощи метода pendingDatagramSize() мы узнаем размер ждущей обработки дейтаграммы. Дейтаграммы всегда высылаются одним блоком, а это
//значит, что при любом ее размере ее нужно считать целиком. Следует учитывать, что если размер буфера окажется недостаточным, то данные
//будут обрезаны. Поэтому в соответствии с полученным размером дейтаграммы мы изменяем размер буфера для считывания дейтаграмм и вызовом
//метода readDatagram() копируем в буфер данные baDatagram. После завершения цикла считываем в поток in дату и время, полученные из
//дейтаграммы, и отображаем их в окне. Для этого вызываем метод QTextEdit::append().
