//Для запуска сервера нам необходимо вызвать в конструкторе метод listen(). В этот метод нужно передать номер порта, который мы получили в
//конструкторе. При возникновении ошибочных ситуаций, например невозможности захвата порта, этот метод возвратит значение false, на которое
//мы отреагируем показом окна сообщения об ошибке. Если ошибки не произошло, мы соединяем определенный нами слот slotNewConnection()с сигналом
//newConnection(), который отправляется при каждом присоединении нового клиента. Для отображения информации мы создаем виджет многострочного
//текстовоо поля(указатель m_ptxt) и устанавливаем в нем режимЮ в котором возможен только просмотр информации, вызовом метода setReadOnly().

MyServer::MeServer(int nPort, QWidget* pwgt /*=0*/) : QWidget(pwgt)
                                                    , m_nNextBlockSize(0)
{
    m_ptcpServer = new QTcpServer(this);
    if (!m_ptcpServer->listen(QHostAddress::Any, nPort)) {
        QMessageBox::critical(0,
                              "Sever Error",
                              "Unable to start the server:"
                              + m_ptcpServer->errorString()
                              );
        m_ptcpServer->close();
        return;
    }
    connect(m_ptcpServer, SIGNAL(newConnection()),
            this,         SLOT(slotNewConnection())
            );

    m_ptxt = new QTextEdit;
    m_ptxt->setReadOnly(true);

    //Layout setup
    QVBoxLayout* pvbxLayout = new QVBoxLayout;
    pvbxLayout->addWidget (new QLabel("<H1>Server</H1>"));
    pvbxLayout->addWidget(m_ptxt);
    setLayout(pvbxLayout);
}


//Метод SlotNewConnection() вызыввается каждый раз при соединении с новым клиентом. Для подтверждения соединения с клиентом необходимо вызвать
//метод nextPendingConnection(), который возвратит сокет, посредством которого можно осуществялть дальнейшую связь с клиентом. Дальнейшая
//работа сокета обеспечивается сигально-слотовыми связями. Мы соединяем сигнал disconnected(), отправляемый сокетом при отсоединении клиента, со
//стандартным слотом QObject::deleteLater(), предназначенным для его последующего уничтожения. При поступлении запросов о клиентов отправляется
//сигнал readyToResd(), который мы соединяем со слотом slotReadClient().

/*virtual*/ void MyServer::slotNewConnection()
{
    QTcpSocket* pClientSocket = m_ptcpServer->nextPendingConnection();
    connect(pClientSocket, SIGNAL(disconnected()),
            pClientSocket, SLOT(deleteLater())
            );
    connect(pClientSocket, SIGNAL(readyRead())
            this,          SLOT(slotReadClient())
            );
    sendToClient(pClientSocket, "Server Response: Connected!");

}


//Здесь сначала выполняется преобразование указателя, возвращаемого методом sender(), к типу QTcpSocket. Цикл for нам нужен потому,
//что не все высланные клиентом данные могут прийти одновременно. Поэтому сервер должен "уметь" получать как весь блок целиком, так и только
//часть блока, а также и все блоки сразу. Каждый переданный сокетом блок начинается полем размера блока. Размер блока считывается при
//условии, что размер полученных данных не меньше двух байт и атрибут m_nNextBlockSize равен нулю (т. е. размер блока неизвестен). Если
//размер доступных для чтения данных больше или равен размеру блока, тогда данные считываются из потока в переменные time и str. Затем
//значение переменной time преобразуется вызовом метода toString() в строку и вместе со строкой str записывается в строку сообщения
//strMessage, которая добавляется в виджет текстового поля вызовом метода append(). Анализ блока данных завершается присваивани-ем
//атрибуту m_nNextBlockSize значения 0, которое говорит о том, что размер очередного блока данных неизвестен. Вызовом метода sendToClient()
//мы сообщаем клиенту о том, что нам успешно удалось прочитать высланные им данные.

void MyServer::slotReadClient()
{
    QTcpSocket* pClientSocket = (QTcpSocket*)sender();
    QDataStream in(pClientSocket);
    in.setVersion(QDataStream::Qt_4_5);
    for (;;) {
        if (!m_nNextBlockSize) {
            if (pClientSocket->bytesAvailable() < sizeof(quint16)) {
                break;
            }
            in >> m_nNextBlockSize;
        }
        if (pClientSocket->bytesAvailable() < m_nNextBlockSize) {
            break;
        }
        QTime time;
        QString str;
        in >> time >> str;

        QString strMessage =
            time.toString() + " " + "Client has sent — " + str;
        m_ptxt->append(strMessage);

        m_nNextBlockSize = 0;

        sendToClient(pClientSocket,
                     "Server Response: Received \"" + str + "\""
                     );

    }
}



//В методе sendToClient() мы формируем данные, которые будут отосланы клиенту. Есть небольшая деталь, заключающаяся в том, что нам заранее
//не известен размер блока, а следовательно, мы не можем записывать данные сразу в сокет, т. к. размер блока должен быть выслан в первую
//очередь. Поэтому прибегаем к следующему трюку. Сначала создаем объект arrBlock класса QByteArray. На его основе создаем объект класса
//QDataStream, в который записываем все данные блока, причем вместо реального размера записываем 0. После этого перемещаем указатель на
//начало блока вызовом метода seek(), вычисляем размер блока как размер arrBlock, уменьшенный на sizeof(quint16), и записываем его в
//поток (out) с текущей позиции, которая уже перемещена в начало блока. После этого созданный блок записывается в сокет вызовом метода
//write().


void MyServer::sendToClient(QTcpSocket* pSocket, const QString& str)
{
    QByteArray arrBlock;
    QDataStream out(&arrBlock, QIODevice::WriteOnly);
    out.setVersion(QDataStream::Qt_4_5);
    out << quint16(0) << QTime::currentTime() << str;

    out.device()->seek(0);
    out << quint16(arrBlock.size() — sizeof(quint16));

    pSocket->write(arrBlock);
}
